==> qtools_profiler-2.2.x-dev.tar.gz: qtools_profiler/src/ReportService.php <==
<?php

namespace Drupal\qtools_profiler;

use Drupal\Core\Render\Renderer;
use Drupal\Core\Url;
use Drupal\Core\Database\Connection;
use Drupal\Core\StringTranslation\StringTranslationTrait;

/**
 * Class ReportService.
 *
 * @package Drupal\qtools_profiler
 */
class ReportService {

  use StringTranslationTrait;

  /**
   * Drupal\Core\Database\Connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $connection;

  /**
   * Renderer.
   *
   * @var Drupal\Core\Render\Renderer
   */
  protected $renderer;

  /**
   * Drupal\qtools_profiler\PerformanceService.
   *
   * @var \Drupal\qtools_profiler\PerformanceService
   */
  protected $performanceService;

  // Table names.
  const ITEMS_PER_PAGE = 15;

  /**
   * Constructor.
   */
  public function __construct(Connection $connection, PerformanceService $performanceService, Renderer $renderer) {
    $this->connection = $connection;
    $this->performanceService = $performanceService;
    $this->renderer = $renderer;
  }

  /**
   * Sets report filter values.
   */
  public function setReportFilters($filters) {
    $_SESSION['qtools_profiler']['reports_form']['filters'] = $filters;
  }

  /**
   * Gets current filter values.
   *
   * @return array
   *   Values of filters.
   */
  public function getReportFilters($use_def = FALSE) {
    $defaults = [
      'start' => strtotime(date('Y-m-d', 0)),
      'end' => strtotime(date('Y-m-d', time()) . ' +1day'),
      'report' => FALSE,
    ];
    if ($use_def) {
      return $defaults;
    }

    $values = !empty($_SESSION['qtools_profiler']['reports_form']['filters']) ?
      $_SESSION['qtools_profiler']['reports_form']['filters'] :
      [];

    foreach ($defaults as $key => $value) {
      if (isset($values[$key])) {
        $defaults[$key] = $values[$key];
      }
    }
    return $defaults;
  }

  /**
   * Sets channels filter values.
   */
  public function setChannelsFilters($filters) {
    $_SESSION['qtools_profiler']['channels_form']['filters'] = $filters;
  }

  /**
   * Gets current channels filter values.
   *
   * @return array
   *   Values of filters.
   */
  public function getChannelsFilters($use_def = FALSE) {
    $defaults = [
      'created' => strtotime('today'),
      'last_accessed' => strtotime('today'),
      'cookie_id' => '',
    ];
    if ($use_def) {
      return $defaults;
    }

    $values = !empty($_SESSION['qtools_profiler']['channels_form']['filters']) ?
      $_SESSION['qtools_profiler']['channels_form']['filters'] :
      [];

    foreach ($defaults as $key => $value) {
      if (isset($values[$key])) {
        $defaults[$key] = $values[$key];
      }
    }
    return $defaults;
  }

  /**
   * Builds a base query for Request report.
   *
   * @return \Drupal\Core\Database\Query\SelectInterface
   *   Select query.
   */
  public function reportRequestBaseQuery($filters = [], $grouping = NULL) {
    // Base table.
    $query = $this->connection->select(PerformanceService::TABLE_REQUEST, 'r');

    // Join additional resources.
    $query->leftJoin(PerformanceService::TABLE_ROUTE, 'route', 'r.rid = route.id');
    $query->leftJoin(PerformanceService::TABLE_RESPONSE, 'response', 'r.id = response.rid');

    // Apply filters.
    $query->condition('r.timestamp', $filters['start'], '>=');
    $query->condition('r.timestamp', $filters['end'], '<=');

    if (!empty($filters['route'])) {
      $query->condition('r.rid', $filters['route'], '=');
    }
    if (!empty($filters['report'])) {
      $query->condition('response.report', 0, '<>');
    }

    // Add grouping.
    $query = $query->groupBy('r.id');

    // Add fields.
    $query->addExpression('MIN(r.id)', 'id');
    $query->addExpression('MIN(r.timestamp)', 'timestamp');
    $query->addExpression('MIN(response.code)', 'code');
    $query->addExpression('MIN(response.report)', 'report');
    $query->addExpression('MIN(r.uri)', 'uri');
    $query->addExpression('MIN(response.time)', 'time');
    $query->addExpression('MIN(response.memory)', 'memory');

    $query = $query->extend('Drupal\Core\Database\Query\TableSortExtender')
      ->orderByHeader($this->reportRequestTableHeader($filters, $grouping));

    $query = $query->extend('Drupal\Core\Database\Query\PagerSelectExtender')
      ->limit(static::ITEMS_PER_PAGE);

    return $query;
  }

  /**
   * Builds a base query for Channels report.
   *
   * @return \Drupal\Core\Database\Query\SelectInterface
   *   Select query.
   */
  public function reportChannelsBaseQuery($filters = [], $grouping = NULL) {
    // Base table.
    $query = $this->connection->select(PerformanceService::TABLE_CHANNEL, 'c');

    // Join additional resources.
    $query->leftJoin(PerformanceService::TABLE_REQUEST, 'request', 'c.id = request.cid');

    // Apply filters.
    $query->condition('c.timestamp', $filters['created'], '>=');
    $query->condition('request.timestamp', $filters['last_accessed'], '>=');

    if (!empty($filters['cookie_id'])) {
      $channel_info = $this->performanceService->getChannelInfo($filters['cookie_id']);
      if (!empty($channel_info)) {
        $channel_id = $this->performanceService->getChannelId($filters['cookie_id'], $channel_info);
        $query->condition('c.id', $channel_id);
      }
    }

    // Add grouping.
    $query = $query->groupBy('c.id');

    // Add fields.
    $query->addExpression('MIN(c.id)', 'id');
    $query->addExpression('MIN(c.label)', 'label');
    $query->addExpression('MIN(c.timestamp)', 'created');
    $query->addExpression('MAX(request.timestamp)', 'last_request');
    $query->addExpression('MAX(request.id)', 'rid');

    $query = $query->extend('Drupal\Core\Database\Query\TableSortExtender')
      ->orderByHeader($this->reportChannelsTableHeader($filters, $grouping));

    $query = $query->extend('Drupal\Core\Database\Query\PagerSelectExtender')
      ->limit(static::ITEMS_PER_PAGE);

    return $query;
  }

  /**
   * Process results of Request report.
   *
   * @return array
   *   Processed items.
   */
  public function reportRequestProcessResults($results, $filters = [], $grouping = NULL) {
    $rows = [];

    // Replace route column with a link to dedicated report.
    foreach ($results as $result) {
      $data = (array) $result;
      $link = [
        '#type' => 'link',
        '#title' => $result->uri,
        '#url' => Url::fromUri('internal:' . $result->uri),
        '#attributes' => [
          'target' => '_blank',
        ],
      ];
      $data['uri'] = $this->renderer->renderRoot($link);

      $data['timestamp'] = date('Y-m-d H:i:s Z', $data['timestamp']);
      if ($result->report) {
        $link = [
          '#type' => 'link',
          '#title' => $data['timestamp'],
          '#url' => Url::fromRoute('qtools_profiler.admin.report_profile', ['id' => $data['id']]),
          '#attributes' => [
            'target' => '_blank',
          ],
        ];
        $data['timestamp'] = $this->renderer->renderRoot($link);
      }

      unset($data['id']);
      unset($data['report']);
      $rows[] = $data;
    }
    return $this->processNumericValues($rows);
  }

  /**
   * Process results of Channels report.
   *
   * @return array
   *   Processed items.
   */
  public function reportChannelsProcessResults($results, $filters = [], $grouping = NULL) {
    $rows = [];
    $date_time_format = 'Y-m-d H:i:s';

    // Replace route column with a link to dedicated report.
    foreach ($results as $result) {
      $query = $this->connection->select(PerformanceService::TABLE_REQUEST, 'r');
      $query->leftJoin('users_field_data', 'ufd', 'r.uid = ufd.uid');
      $query->fields('r', []);
      $query->fields('ufd', ['name']);

      $query->condition('id', $result->rid);
      $last_request = $query->execute()->fetchAll()[0];

      $link = [
        '#type' => 'link',
        '#title' => $result->label ?? $last_request->name,
        '#url' => Url::fromRoute('qtools_profiler.admin.channel_view', ['id' => $result->id]),
        '#attributes' => [
          'target' => '_blank',
        ],
      ];
      $id_link = $this->renderer->renderRoot($link);

      $rows[] = [
        'id' => $id_link,
        'created' => date($date_time_format, $result->created),
        'last_accessed' => date($date_time_format, $result->last_request),
        'user' => $last_request->name,
      ];
    }
    return $this->processNumericValues($rows);
  }

  /**
   * Return header definition for requests.
   *
   * @return array
   *   Table header.
   */
  public function reportRequestTableHeader($filters = [], $grouping = NULL) {
    return [
      'timestamp' => [
        'data' => $this->t('timestamp'),
        'field' => 'timestamp',
        'sort' => 'desc',
      ],
      'id' => [
        'data' => $this->t('code'),
        'field' => 'code',
      ],
      'uri' => [
        'data' => $this->t('uri'),
        'field' => 'uri',
      ],
      'time' => [
        'data' => $this->t('time'),
        'field' => 'time',
      ],
      'memory' => [
        'data' => $this->t('memory'),
        'field' => 'memory',
      ],
    ];
  }

  /**
   * Return header definition for channels.
   *
   * @return array
   *   Table header.
   */
  public function reportChannelsTableHeader($filters = [], $grouping = NULL) {
    return [
      'id' => [
        'data' => $this->t('ID'),
        'field' => 'id',
        'sort' => 'desc',
      ],
      'created' => [
        'data' => $this->t('Created'),
        'field' => 'created',
      ],
      'last_accessed' => [
        'data' => $this->t('Last accessed'),
        'field' => 'last_accessed',
      ],
      'user' => [
        'data' => $this->t('User'),
      ],
    ];
  }

  /**
   * Apply precision to numeric values.
   */
  protected function processNumericValues($rows) {
    foreach ($rows as $row => $data) {
      foreach ($data as $key => $value) {
        if (is_numeric($value)) {
          $data[$key] = round($value, 2);
        }
      }
      $rows[$row] = $data;
    }

    return $rows;
  }

  /**
   * Apply precision to numeric values.
   *
   * @return \Drupal\Core\Database\Query\SelectInterface
   *   Query with filters applied.
   */
  protected function applyFilters($query, $filters = []) {
    foreach ($filters as $field => $value) {
      $query->condition($field, $value);
    }
    return $query;
  }

  /**
   * Get profiling run details.
   */
  public function getProfileDetails($id) {
    $file_name = $this->performanceService->getProfilingDataFileName($id);
    if (!empty($file_name) && file_exists($file_name)) {
      $content = unserialize(file_get_contents($file_name));
    }
    else {
      $content = NULL;
    }
    return $content;
  }

}

==> qtools_profiler-3.0.x-dev.tar.gz: qtools_profiler/src/ReportService.php <==
<?php

namespace Drupal\qtools_profiler;

use Drupal\Core\Render\Renderer;
use Drupal\Core\Url;
use Drupal\Core\Database\Connection;
use Drupal\Core\StringTranslation\StringTranslationTrait;

/**
 * Class ReportService.
 *
 * @package Drupal\qtools_profiler
 */
class ReportService {

  use StringTranslationTrait;

  // Table names.
  const ITEMS_PER_PAGE = 15;

  /**
   * Constructor.
   */
  public function __construct(
    protected Connection $connection,
    protected PerformanceService $performanceService,
    protected Renderer $renderer,
  ) {
  }

  /**
   * Sets report filter values.
   */
  public function setReportFilters($filters) {
    $_SESSION['qtools_profiler']['reports_form']['filters'] = $filters;
  }

  /**
   * Gets current filter values.
   *
   * @return array
   *   Values of filters.
   */
  public function getReportFilters($use_def = FALSE) {
    $defaults = [
      'start' => strtotime(date('Y-m-d', 0)),
      'end' => strtotime(date('Y-m-d', time()) . ' +1day'),
      'report' => FALSE,
    ];
    if ($use_def) {
      return $defaults;
    }

    $values = !empty($_SESSION['qtools_profiler']['reports_form']['filters']) ?
      $_SESSION['qtools_profiler']['reports_form']['filters'] :
      [];

    foreach ($defaults as $key => $value) {
      if (isset($values[$key])) {
        $defaults[$key] = $values[$key];
      }
    }
    return $defaults;
  }

  /**
   * Sets channels filter values.
   */
  public function setChannelsFilters($filters) {
    $_SESSION['qtools_profiler']['channels_form']['filters'] = $filters;
  }

  /**
   * Gets current channels filter values.
   *
   * @return array
   *   Values of filters.
   */
  public function getChannelsFilters($use_def = FALSE) {
    $defaults = [
      'created' => strtotime('today'),
      'last_accessed' => strtotime('today'),
      'cookie_id' => '',
    ];
    if ($use_def) {
      return $defaults;
    }

    $values = !empty($_SESSION['qtools_profiler']['channels_form']['filters']) ?
      $_SESSION['qtools_profiler']['channels_form']['filters'] :
      [];

    foreach ($defaults as $key => $value) {
      if (isset($values[$key])) {
        $defaults[$key] = $values[$key];
      }
    }
    return $defaults;
  }

  /**
   * Builds a base query for Request report.
   *
   * @return \Drupal\Core\Database\Query\SelectInterface
   *   Select query.
   */
  public function reportRequestBaseQuery($filters = [], $grouping = NULL) {
    // Base table.
    $query = $this->connection->select(PerformanceService::TABLE_REQUEST, 'r');

    // Join additional resources.
    $query->leftJoin(PerformanceService::TABLE_ROUTE, 'route', 'r.rid = route.id');
    $query->leftJoin(PerformanceService::TABLE_RESPONSE, 'response', 'r.id = response.rid');

    // Apply filters.
    $query->condition('r.timestamp', $filters['start'], '>=');
    $query->condition('r.timestamp', $filters['end'], '<=');

    if (!empty($filters['route'])) {
      $query->condition('r.rid', $filters['route'], '=');
    }
    if (!empty($filters['report'])) {
      $query->condition('response.report', 0, '<>');
    }

    // Add grouping.
    $query = $query->groupBy('r.id');

    // Add fields.
    $query->addExpression('MIN(r.id)', 'id');
    $query->addExpression('MIN(r.timestamp)', 'timestamp');
    $query->addExpression('MIN(response.code)', 'code');
    $query->addExpression('MIN(response.report)', 'report');
    $query->addExpression('MIN(r.uri)', 'uri');
    $query->addExpression('MIN(response.time)', 'time');
    $query->addExpression('MIN(response.memory)', 'memory');

    $query = $query->extend('Drupal\Core\Database\Query\TableSortExtender')
      ->orderByHeader($this->reportRequestTableHeader($filters, $grouping));

    $query = $query->extend('Drupal\Core\Database\Query\PagerSelectExtender')
      ->limit(static::ITEMS_PER_PAGE);

    return $query;
  }

  /**
   * Builds a base query for Channels report.
   *
   * @return \Drupal\Core\Database\Query\SelectInterface
   *   Select query.
   */
  public function reportChannelsBaseQuery($filters = [], $grouping = NULL) {
    // Base table.
    $query = $this->connection->select(PerformanceService::TABLE_CHANNEL, 'c');

    // Join additional resources.
    $query->leftJoin(PerformanceService::TABLE_REQUEST, 'request', 'c.id = request.cid');

    // Apply filters.
    $query->condition('c.timestamp', $filters['created'], '>=');
    $query->condition('request.timestamp', $filters['last_accessed'], '>=');

    if (!empty($filters['cookie_id'])) {
      $channel_info = $this->performanceService->getChannelInfo($filters['cookie_id']);
      if (!empty($channel_info)) {
        $channel_id = $this->performanceService->getChannelId($filters['cookie_id'], $channel_info);
        $query->condition('c.id', $channel_id);
      }
    }

    // Add grouping.
    $query = $query->groupBy('c.id');

    // Add fields.
    $query->addExpression('MIN(c.id)', 'id');
    $query->addExpression('MIN(c.label)', 'label');
    $query->addExpression('MIN(c.timestamp)', 'created');
    $query->addExpression('MAX(request.timestamp)', 'last_request');
    $query->addExpression('MAX(request.id)', 'rid');

    $query = $query->extend('Drupal\Core\Database\Query\TableSortExtender')
      ->orderByHeader($this->reportChannelsTableHeader($filters, $grouping));

    $query = $query->extend('Drupal\Core\Database\Query\PagerSelectExtender')
      ->limit(static::ITEMS_PER_PAGE);

    return $query;
  }

  /**
   * Process results of Request report.
   *
   * @return array
   *   Processed items.
   */
  public function reportRequestProcessResults($results, $filters = [], $grouping = NULL) {
    $rows = [];

    // Replace route column with a link to dedicated report.
    foreach ($results as $result) {
      $data = (array) $result;
      $link = [
        '#type' => 'link',
        '#title' => $result->uri,
        '#url' => Url::fromUri('internal:' . $result->uri),
        '#attributes' => [
          'target' => '_blank',
        ],
      ];
      $data['uri'] = $this->renderer->renderRoot($link);

      $data['timestamp'] = date('Y-m-d H:i:s Z', $data['timestamp']);
      if ($result->report) {
        $link = [
          '#type' => 'link',
          '#title' => $data['timestamp'],
          '#url' => Url::fromRoute('qtools_profiler.admin.report_profile', ['id' => $data['id']]),
          '#attributes' => [
            'target' => '_blank',
          ],
        ];
        $data['timestamp'] = $this->renderer->renderRoot($link);
      }

      unset($data['id']);
      unset($data['report']);
      $rows[] = $data;
    }
    return $this->processNumericValues($rows);
  }

  /**
   * Process results of Channels report.
   *
   * @return array
   *   Processed items.
   */
  public function reportChannelsProcessResults($results, $filters = [], $grouping = NULL) {
    $rows = [];
    $date_time_format = 'Y-m-d H:i:s';

    // Replace route column with a link to dedicated report.
    foreach ($results as $result) {
      $query = $this->connection->select(PerformanceService::TABLE_REQUEST, 'r');
      $query->leftJoin('users_field_data', 'ufd', 'r.uid = ufd.uid');
      $query->fields('r', []);
      $query->fields('ufd', ['name']);

      $query->condition('id', $result->rid);
      $last_request = $query->execute()->fetchAll()[0];

      $link = [
        '#type' => 'link',
        '#title' => $result->label ?? $last_request->name,
        '#url' => Url::fromRoute('qtools_profiler.admin.channel_view', ['id' => $result->id]),
        '#attributes' => [
          'target' => '_blank',
        ],
      ];
      $id_link = $this->renderer->renderRoot($link);

      $rows[] = [
        'id' => $id_link,
        'created' => date($date_time_format, $result->created),
        'last_accessed' => date($date_time_format, $result->last_request),
        'user' => $last_request->name,
      ];
    }
    return $this->processNumericValues($rows);
  }

  /**
   * Return header definition for requests.
   *
   * @return array
   *   Table header.
   */
  public function reportRequestTableHeader($filters = [], $grouping = NULL) {
    return [
      'timestamp' => [
        'data' => $this->t('timestamp'),
        'field' => 'timestamp',
        'sort' => 'desc',
      ],
      'id' => [
        'data' => $this->t('code'),
        'field' => 'code',
      ],
      'uri' => [
        'data' => $this->t('uri'),
        'field' => 'uri',
      ],
      'time' => [
        'data' => $this->t('time'),
        'field' => 'time',
      ],
      'memory' => [
        'data' => $this->t('memory'),
        'field' => 'memory',
      ],
    ];
  }

  /**
   * Return header definition for channels.
   *
   * @return array
   *   Table header.
   */
  public function reportChannelsTableHeader($filters = [], $grouping = NULL) {
    return [
      'id' => [
        'data' => $this->t('ID'),
        'field' => 'id',
        'sort' => 'desc',
      ],
      'created' => [
        'data' => $this->t('Created'),
        'field' => 'created',
      ],
      'last_accessed' => [
        'data' => $this->t('Last accessed'),
        'field' => 'last_accessed',
      ],
      'user' => [
        'data' => $this->t('User'),
      ],
    ];
  }

  /**
   * Apply precision to numeric values.
   */
  protected function processNumericValues($rows) {
    foreach ($rows as $row => $data) {
      foreach ($data as $key => $value) {
        if (is_numeric($value)) {
          $data[$key] = round($value, 2);
        }
      }
      $rows[$row] = $data;
    }

    return $rows;
  }

  /**
   * Apply precision to numeric values.
   *
   * @return \Drupal\Core\Database\Query\SelectInterface
   *   Query with filters applied.
   */
  protected function applyFilters($query, $filters = []) {
    foreach ($filters as $field => $value) {
      $query->condition($field, $value);
    }
    return $query;
  }

  /**
   * Get profiling run details.
   */
  public function getProfileDetails($id) {
    $file_name = $this->performanceService->getProfilingDataFileName($id);
    if (!empty($file_name) && file_exists($file_name)) {
      $content = unserialize(file_get_contents($file_name));
    }
    else {
      $content = NULL;
    }
    return $content;
  }

}
